@@ -7,61 +7,78 @@
 //! puffin_egui::profiler_window(&egui_ctx);
 //! ```
 
-#![cfg_attr(not(debug_assertions), deny(warnings))] // Forbid warnings in release builds
-#![deny(broken_intra_doc_links)]
-#![deny(invalid_codeblock_attributes)]
-#![deny(private_intra_doc_links)]
-#![forbid(unsafe_code)]
+// BEGIN - Embark standard lints v0.4
+// do not change or add/remove here, but one can add exceptions after this section
+// for more info see: <https://github.com/EmbarkStudios/rust-ecosystem/issues/59>
+#![deny(unsafe_code)]
 #![warn(
     clippy::all,
     clippy::await_holding_lock,
+    clippy::char_lit_as_u8,
+    clippy::checked_conversions,
     clippy::dbg_macro,
     clippy::debug_assert_with_mut_call,
     clippy::doc_markdown,
     clippy::empty_enum,
     clippy::enum_glob_use,
     clippy::exit,
+    clippy::expl_impl_clone_on_copy,
+    clippy::explicit_deref_methods,
     clippy::explicit_into_iter_loop,
+    clippy::fallible_impl_from,
     clippy::filter_map_next,
+    clippy::float_cmp_const,
     clippy::fn_params_excessive_bools,
     clippy::if_let_mutex,
+    clippy::implicit_clone,
     clippy::imprecise_flops,
     clippy::inefficient_to_string,
+    clippy::invalid_upcast_comparisons,
     clippy::large_types_passed_by_value,
     clippy::let_unit_value,
     clippy::linkedlist,
     clippy::lossy_float_literal,
     clippy::macro_use_imports,
+    clippy::manual_ok_or,
     clippy::map_err_ignore,
     clippy::map_flatten,
+    clippy::map_unwrap_or,
     clippy::match_on_vec_items,
     clippy::match_same_arms,
     clippy::match_wildcard_for_single_variants,
     clippy::mem_forget,
     clippy::mismatched_target_os,
-    clippy::missing_errors_doc,
-    clippy::missing_safety_doc,
+    clippy::mut_mut,
+    clippy::mutex_integer,
     clippy::needless_borrow,
     clippy::needless_continue,
-    clippy::needless_pass_by_value,
     clippy::option_option,
-    clippy::pub_enum_variant_names,
+    clippy::path_buf_push_overwrite,
+    clippy::ptr_as_ptr,
     clippy::ref_option_ref,
     clippy::rest_pat_in_fully_bound_structs,
+    clippy::same_functions_in_if_condition,
+    clippy::semicolon_if_nothing_returned,
     clippy::string_add_assign,
     clippy::string_add,
+    clippy::string_lit_as_bytes,
     clippy::string_to_string,
     clippy::todo,
+    clippy::trait_duplication_in_bounds,
     clippy::unimplemented,
     clippy::unnested_or_patterns,
     clippy::unused_self,
+    clippy::useless_transmute,
     clippy::verbose_file_reads,
+    clippy::zero_sized_map_values,
     future_incompatible,
     nonstandard_style,
     rust_2018_idioms
 )]
-#![allow(clippy::float_cmp)]
-#![allow(clippy::manual_range_contains)]
+// END - Embark standard lints v0.4
+// crate-specific exceptions:
+#![deny(missing_crate_level_docs)]
+#![allow(clippy::float_cmp, clippy::manual_range_contains)]
 
 pub use {egui, puffin};
 
@@ -95,7 +112,7 @@ static PROFILE_UI: once_cell::sync::Lazy<Mutex<ProfilerUi>> =
 /// Call this from within an [`egui::Window`], or use [`profiler_window`] instead.
 pub fn profiler_ui(ui: &mut egui::Ui) {
     let mut profile_ui = PROFILE_UI.lock().unwrap();
-    profile_ui.ui(ui)
+    profile_ui.ui(ui);
 }
 
 // ----------------------------------------------------------------------------
@@ -304,6 +321,14 @@ fn latest_frames() -> Frames {
 }
 
 impl ProfilerUi {
+    pub fn reset(&mut self) {
+        let options = self.options;
+        *self = Self {
+            options,
+            ..Default::default()
+        };
+    }
+
     /// Show an [`egui::Window`] with the profiler contents.
     ///
     /// If you want to control the window yourself, use [`Self::ui`] instead.
@@ -423,7 +448,8 @@ impl ProfilerUi {
                 );
             ui.separator();
             ui.label(format!(
-                "Current frame: {:.1} ms, {} threads, {} scopes, {:.1} kB",
+                "Showing frame #{}, {:.1} ms, {} threads, {} scopes, {:.1} kB",
+                frame.frame_index,
                 (max_ns - min_ns) as f64 * 1e-6,
                 frame.thread_streams.len(),
                 frame.num_scopes,
@@ -431,9 +457,8 @@ impl ProfilerUi {
             ));
         });
 
-        if frame.thread_streams.len() > 1 {
-            self.options.sorting.ui(ui);
-        }
+        // The number of threads can change between frames, so always show this even if there currently is only one thread:
+        self.options.sorting.ui(ui);
 
         if self.paused.is_none() {
             ui.ctx().request_repaint(); // keep refreshing to see latest data
@@ -570,25 +595,16 @@ impl ProfilerUi {
 
         let longest_count = frames.recent.len().max(frames.slowest.len());
 
-        // TODO: in egui 0.14, use `egui::Grid::new("frame_grid").num_columns(2)`
+        egui::Grid::new("frame_grid").num_columns(2).show(ui, |ui| {
+            ui.label("Recent:");
 
-        let label_size = egui::vec2(48.0, self.options.frame_list_height);
-        let vertical = egui::Layout::top_down(egui::Align::LEFT);
-
-        ui.horizontal(|ui| {
-            ui.allocate_ui_with_layout(label_size, vertical, |ui| {
-                ui.set_min_size(label_size);
-                ui.add_space(24.0); // make it a bit more centered
-                ui.label("Recent:");
-            });
             Frame::dark_canvas(ui.style()).show(ui, |ui| {
                 self.show_frame_list(ui, &frames.recent, longest_count, &mut hovered_frame);
             });
-        });
 
-        ui.horizontal(|ui| {
-            ui.allocate_ui_with_layout(label_size, vertical, |ui| {
-                ui.set_min_size(label_size);
+            ui.end_row();
+
+            ui.vertical(|ui| {
                 ui.style_mut().wrap = Some(false);
                 ui.add_space(16.0); // make it a bit more centered
                 ui.label("Slowest:");
@@ -1027,7 +1043,7 @@ fn paint_merge_scope(
 
         if result == PaintResult::Hovered {
             egui::show_tooltip_at_pointer(&info.ctx, Id::new("puffin_profiler_tooltip"), |ui| {
-                merge_scope_tooltip(ui, merge)
+                merge_scope_tooltip(ui, merge);
             });
         }
     }
